package pwc.taxtech.atms.common.ftp;

import org.apache.commons.lang3.StringUtils;
import org.apache.commons.net.ftp.FTPClient;
import org.apache.commons.pool2.impl.GenericObjectPool;
import org.apache.commons.pool2.impl.GenericObjectPoolConfig;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;


import javax.annotation.PostConstruct;
import java.io.IOException;
import java.io.InputStream;

@Component
public class FTPClientPool {
    private GenericObjectPoolConfig config;
    private GenericObjectPool<FTPClient> pool;
    private String ftpRootPath;
    private static final String SYMBOL = "/";

    @Value("${ftp.host}")
    private String ftpHost;
    @Value("${ftp.port}")
    private Integer ftpPort;
    @Value("${ftp.user}")
    private String ftpUser;
    @Value("${ftp.pwd}")
    private String ftpPwd;

    @PostConstruct
    public void init() throws Exception {
        config = new GenericObjectPoolConfig();
        FTPClientConfig clientConfig = new FTPClientConfig();
        clientConfig.setHost(ftpHost);
        clientConfig.setPort(ftpPort);
        clientConfig.setUsername(ftpUser);
        clientConfig.setPassword(ftpPwd);
        pool = new GenericObjectPool<>(new FtpClientFactory(clientConfig), config);
        ftpRootPath = pool.borrowObject().printWorkingDirectory();
    }

    public FTPClient getClient() throws Exception {
        return pool.borrowObject();
    }

    /**
     * 上传
     *
     * @param filePath    相对路径
     * @param fileName    文件名
     * @param inputStream InputStream
     * @throws Exception Exception
     */
    public void upload(String filePath, String fileName, InputStream inputStream) throws Exception {
        String upPath;
        if (StringUtils.isBlank(filePath)) {
            upPath = ftpRootPath;
        } else {
            upPath = filePath;
        }
        if (!StringUtils.endsWith(upPath, SYMBOL)) {
            upPath = upPath + SYMBOL;
        }
        FTPClient client = getClient();
        if (!isExist(upPath, client)) {
            mkDir(upPath, client);
        }
        client.storeFile(upPath + fileName, inputStream);
    }

    /**
     * 下载
     *
     * @param filePath 相对路径 + 文件名
     * @return InputStream
     * @throws Exception Exception
     */
    public InputStream download(String filePath) throws Exception {
        FTPClient client = getClient();
        client.changeWorkingDirectory(ftpRootPath);
        return StringUtils.isBlank(filePath) ? null : client.retrieveFileStream(filePath);
    }

    private void mkDir(String path, FTPClient ftpClient) throws IOException {
        if (StringUtils.isNotBlank(path)) {
            ftpClient.changeWorkingDirectory(ftpRootPath);
            String[] paths = path.split("/");
            for (String p : paths) {
                if (StringUtils.isNotBlank(p) && !StringUtils.equals(p, ".")) {
                    if (!ftpClient.changeWorkingDirectory(p)) {
                        ftpClient.makeDirectory(p);
                        if (!ftpClient.changeWorkingDirectory(p)) {
                            throw new IOException("changeWorkingDirectory error.");
                        }
                    }
                }
            }
        }
    }

    private boolean isExist(String path, FTPClient ftpClient) throws IOException {
        String pwd = ftpClient.printWorkingDirectory();
        try {
            return ftpClient.changeWorkingDirectory(path);
        } finally {
            ftpClient.changeWorkingDirectory(pwd);
        }
    }

    public String getFtpRootPath() {
        return ftpRootPath;
    }

    /**
     * 删除ftp文件
     *
     * @param filePath
     */
    public void delete(String filePath) throws Exception {
        if (StringUtils.isBlank(filePath)) {
            return;
        }

        FTPClient client = getClient();
        if (!isExist(filePath, client)) {
            return;
        } else {
            client.deleteFile(filePath);
        }
    }
}
